; Copyright (c) 1992, Joel Krajden.  All Rights Reserved.
;
; Permission to re-distribute all or part of this script is granted,
; provided that this statement, including the above copyright notice, is not
; removed.  You may add your own copyright notice to secure copyright
; protection for new matter that you add to this script.

; I would appreciate any comments

;PopUp is faster and smaller than Paradox's PopUp2 procedure
; but is limited by the number of items that can be listed
; - can be fixed by using multiple strings and string arithmetic

;PopUp uses a buffered string which is displayed in a box using the paintcanvas
;command which wraps inside the box


;Procedures for JKPopUp

Proc SetPopUp(MenuTbl.a,Fld.a,MaxWR.s,SetUpScript.a)
;MenuTbl.a - table holding menu or popup items indexed on Fld.a
;Fld.a - Field name of holding items
;MaxWR.s - maximum window rows
;SetUpScript.a - Script containing initialization code for popup
; to be generated by SetPopUp (8 char no suffix)
;SetPopUp only needs to be run once or if MenuTbl.a is changed


private
  MStr.a, ; concatenated menu string

  ;Box characters
  TLC.a,; = "U"  top left corner
  TRC.a,; = "8"  top right corner
  BLC.a,; = "T"  bot left corner
  BRC.a,; = ">"  bot right corner
  HR.a,; = "M"    horizontal row char
  VC.a,; = "3"    vertical col char

  ;Inside box
  ; WR.s ; max Window rows as determined in script
  WW.s, ; Window width set in script
  HFill.a,; = fill(HR.a,WW.s)
  VFill.a,; = Fill(VC.a,2*WR.s)
  Box.a,; Window
  SetBlink.s,;=207 ;79+128  attribute white on red
  RBotPos.s,;=int((LenMStr.s+.1)/WW.s)
  WR.s,; = Min(RBotPos.s,MaxWR.s)
  MaxWStr.s,;=WR.s*WW.s
  LenMStr.s,;=len(MStr.a)
  MStrEndPos.s,; = LenMStr.s - MaxWStr.s +1

  StrAdj.s.;=WW.s-1

View MenuTbl.a
WW.s = 0 ;MaxWindowWith required
moveto Field Fld.a
scan
  WW.s = Max(WW.s,len([]))
endscan
WW.s = WW.s+2
MStr.a = ""
scan
  MStr.a = MStr.a + [] + Spaces(WW.s-len([])) ;buffer  with at lease 2 spaces
endscan
LenMStr.s=len(MStr.a)
If LenMStr.s > 255
  then
  Message "Total menu string length is > 255 characters. SetPopUp() failed."
  beep beep sleep 5000
  return
else
  Do_It!
  Clearall
endif

RBotPos.s=int((LenMStr.s+.1)/WW.s)
WR.s = Min(RBotPos.s,max(17,MaxWR.s)) ; window rows
MaxWStr.s=WR.s*WW.s
MStrEndPos.s = LenMStr.s - MaxWStr.s +1
StrAdj.s=WW.s-1

TLC.a = "U"
TRC.a = "8"
BLC.a = "T"
BRC.a = ">"
HR.a = "M"
VC.a = "3"; \179

;box.a
HFill.a = fill(HR.a,WW.s)
VFill.a = Fill(VC.a,2*WR.s)
Box.a = TLC.a+HFill.a+TRC.a+VFill.a+BLC.a+HFill.a+BRC.a
SetBlink.s=207

If isfile(SetUpScript.a+".sc")
  then
  {Tools} {Delete} {Script} Select SetUpScript.a {OK}
endif
Print file SetUpScript.a + ".sc"  ; keep length of line within Pdox script limits
"MStr.a = \"",substr(MStr.a,1,85),"\"\n",
 "+\"",substr(MStr.a,86,90),"\"\n",
 "+\"",substr(MStr.a,176,79),"\"\n",
"WW.s = ",WW.s,"\n",
"Box.a = \"",Box.a,"\"\n",
"SetBlink.s = ",SetBlink.s,"\n",
"RBotPos.s = ",RBotPos.s,"\n",
"WR.s = ",WR.s,"\n",
"MaxWStr.s = ",MaxWStr.s,"\n",
"LenMStr.s = ",LenMStr.s,"\n",
"MStrEndPos.s = ",MStrEndPos.s,"\n",
"StrAdj.s = ",StrAdj.s
endproc
writelib applib SetPopUp


Proc Closed PopUp(SetUpScript.a,UseCurPos.l,R.s,C.s,Prompt.a,RetType.s)
;UseCursorPos.l is True,False
; R.s,C.s  top left corner of window if UseCursorPos.l is False
  ;R.s = Max(0,min(24,R.s)) for error checking
  ;C.s = Max(1,min(79,C.s)); make sure there is room for shadow
; RetType.s if 1 return RCurPos.s as num
;              2 return string at RCurPos.s
;              3 return strval(RCurPos.s) + "\255" + string at RCurPos.s
; use   Temp.v = match(Retval,"..\255..",Temp1.a,Temp2.a) if RetType = 3
; return False if Esc used to exit

Play SetUpScript.a ;Initialize vars for PopUp
If UseCurPos.l ; autoadjust placement of popup
  then
  Synccursor
  R.s = min(24,Row()+WR.s+6)-(WR.s+6)
  C.s = min(79,Col()+WW.s+6)-(WW.s+6)
endif
Cursor off
Canvas off

PaintCanvas Fill
 "  >> "+format("W70,AC",Prompt.a) + " <<  " +
 "  >>> "+format("W68,AC","\17DDY to select item, Esc to cancel.") + " <<<  "
 attribute 79 0,0,1,79   ;white on red

Paintcanvas Border attribute 7 R.s+1,C.s+1,R.s+WR.s+2,C.s+WW.s+2
;paints shadow

Paintcanvas Border Fill Box.a attribute 127 R.s,C.s,R.s+WR.s+1,C.s+WW.s+1
; paints window white on grey

Paintcanvas Fill substr(MStr.a,1,MaxWStr.s) attribute 127 R.s+1,C.s+1,R.s+WR.s,C.s+WW.s
;Paints popupmenu

Paintcanvas attribute SetBlink.s R.s+1,C.s+1,R.s+1,C.s+WW.s ;Home set blink attribute
; initialize cursor to home pos

Canvas on
WCurPos.s=1
RCurPos.s=1
HomeCurPos.l = True ;(RCurPos.s=1)
EndCurPos.l = False ;(RCurPos.s<>RBotPos.s)
;D.s is cursor displacment -1,0,1, > 1

While true
Char.s = GetChar()
ACharOk.l = (char.s > 64) ; char.s is not a movment key prepare to search and jump
If ACharOk.l
  then
  AChar.s = asc(Upper(chr(char.s)))
endif
switch
   case Char.s = -80 and not(EndCurPos.l) : D.s=1 ;move down 1 row
   case Char.s = -72 and not(HomeCurPos.l) : D.s = -1  ; move up 1 row
    Case Char.s = 13: ;enter
      Switch
        case RetType.s = 1: return RCurPos.s
        case RetType.s > 1:
          If RetType.s = 3 ; return both pos and string
            then StrRCP.a = Strval(RCurPos.s) + "\255"
          else
            StrRCP.a = ""
          endif
         Temp.l=match(substr(MStr.a,RCurPos.s*WW.s - StrAdj.s,WW.s),"..  ..",Str.a) ; search for 2 spaces
     endswitch
     return StrRCP.a + Str.a ; covers RetType.s = 2
    case Char.s = -71 and not(HomeCurPos.l): ; Home
      RCurPos.s = 1
      WCurPos.s = 1
      D.s = 0
    case Char.s = -79 and not(EndCurPos.l): ; End
      RCurPos.s = RBotPos.s
      WCurPos.s = WR.s
      D.s = 0
    case Char.s = 27 : Return False ; Esc
    case ACharOk.l and AChar.s <91 :
      CurrentStrPos.s = RCurPos.s*WW.s ; at (last character, a space) of RCurPos.s
      NextStrPos.s = Search("  "+Chr(AChar.s),(substr(MStr.a,CurrentStrPos.s-1,LenMStr.s)))
      ;backup from CurrentStrPos.s 1 space so at least 2 spaces exist to start search from
      ; remember buffering of items was with at least 2 spaces
    If NextStrPos.s = 0
      then beep beep loop ; not found
    else
      D.s = int((NextStrPos.s+.1)/WW.s)+1 ; jump D.s cursor positions
      RePaint.l = (D.s >= (WR.s-WCurPos.s)+1) ;move to last row and repaint top to bot
    endif
otherwise : beep beep loop
endswitch

;reset WCurPos attribute
Paintcanvas attribute 127 R.s+WCurPos.s,C.s+1,R.s+WCurPos.s,C.s+WW.s ;reset attribute


WCurPos.s = min(max(WCurPos.s + D.s,1),WR.s)
RCurPos.s = RCurPos.s + D.s
HomeCurPos.l = (RCurPos.s=1)
EndCurPos.l = (RCurPos.s=RBotPos.s)

Switch
  Case D.s =   1:
    If (WCurPos.s = WR.s) and (RCurPos.s > WR.s)  ;scroll down
      then
      Paintcanvas Fill substr(MStr.a,(RCurPos.s-WR.s+1)*WW.s - StrAdj.s,MaxWStr.s) attribute 127 R.s+1,C.s+1,R.s+WR.s,C.s+WW.s
    endif
  Case D.s = -1:
    If WCurPos.s = 1  ;scroll up
      then
      Paintcanvas Fill substr(MStr.a,RCurPos.s*WW.s - StrAdj.s,MaxWStr.s) attribute 127 R.s+1,C.s+1,R.s+WR.s,C.s+WW.s
    endif
  Case D.s = 0 :
      If RCurPos.s >  WR.s  ; end redisplay
        then  StartPos.s = MStrEndPos.s
      else
        StartPos.s = 1
      endif
      Paintcanvas Fill substr(MStr.a,StartPos.s,MaxWStr.s) attribute 127 R.s+1,C.s+1,R.s+WR.s,C.s+WW.s

  Case D.s > 1 and RePaint.l :; search succeeded scroll display
      WCurPos.s = WR.s ; move to last row
      StartPos.s = RCurPos.s*WW.s - MaxWStr.s +1
      Paintcanvas Fill substr(MStr.a,StartPos.s,MaxWStr.s) attribute 127 R.s+1,C.s+1,R.s+WR.s,C.s+WW.s
endswitch


If HomeCurPos.l or EndCurPos.l
  then Attrb.s = SetBlink.s ; blink if home or end
  else Attrb.s=79 ; set window cursor pos  white on red
endif
Paintcanvas attribute Attrb.s R.s+WCurPos.s,C.s+1,R.s+WCurPos.s,C.s+WW.s
endwhile
endproc
writelib applib PopUp
